package pay

import (
	"crypto/md5"
	"encoding/hex"
	"encoding/json"
	"errors"
	"strconv"
	"strings"
	"time"

	"git.tetele.net/tgo/crypter"

	"github.com/golang/protobuf/proto"
)

type PayToBusinessReq struct {
	SiteId        string  `json:"site_id"`
	Dbname        string  `json:"dbname"`
	UserId        string  `json:"user_id"`
	BusinessId    string  `json:"business_id"`
	BalanceTypeId string  `json:"balance_type"`
	Money         float64 `json:"money"`
	Payer         string  `json:"payer"`
	Memo          string  `json:"memo"`
	PayType       string  `json:"pay_type"`
}

type PayToBusinessRes struct {
	Id string
	Sn string
}

const DES_KEY string = "paypayda"

/**
 * 向商家支付
 * 请求及回均加密验签
 * 2021/02/18
 * GZ
 */
func Pay(req PayToBusinessReq, url ...string) (*PayToBusinessRes, error) {

	var pay_rpc_url string = "127.0.0.1:7967"
	if len(url) > 0 && url[0] != "" {
		pay_rpc_url = url[0]
	}
	conn, _, err := DialSiteService("tcp", pay_rpc_url)
	if err != nil {
		return nil, err
	}
	defer conn.Close()

	data_json, err := json.Marshal(req)
	if err != nil {
		return nil, err
	}
	now_int64 := time.Now().Unix()

	encryData := crypter.DesEn(string(data_json), DES_KEY)

	now := strconv.FormatInt(now_int64, 10)

	sign := Sign(encryData, now)

	req := &PayRequest{proto.String(encryData), proto.String(now), proto.String(sign), nil}

	res := &PayResponse{}

	err = conn.PayToBusiness(req, res)

	if err != nil {
		return nil, err
	}

	res_data := res.GetData()

	if res_data != "" {

		time_int64, err := strconv.ParseInt(res.GetTime(), 10, 64)
		if err != nil {
			return nil, err
		}

		now_int64 = time.Now().Unix()

		if now_int64-time_int64 > 10 || time_int64-now_int64 > 10 {
			//时间误差前后10秒，返回
			return nil, errors.New("返回时间错误")
		}

		check_sign := CheckSign(res.GetSign(), res_data, res.GetTime())
		if !check_sign {
			return nil, errors.New("返回数据签名错误")
		}

		//解密
		res_data_de := crypter.DesDe(res_data, DES_KEY)

		var res_arr PayToBusinessRes

		err = json.Unmarshal([]byte(res_data_de), &res_arr)

		if err != nil {
			return nil, err
		}
		return &res_arr, nil
	}

	return nil, nil
}

/**
 * 签名
 */
func Sign(data string, salt string) string {

	var build strings.Builder

	build.WriteString(data)
	build.WriteString(salt)
	build.WriteString("pay66sign33")

	data_str := build.String()

	h := md5.New()
	h.Write([]byte(data_str))             // 需要加密的字符串
	return hex.EncodeToString(h.Sum(nil)) // 输出加密结果

}

/**
 * 验证签名
 */
func CheckSign(sign_str, data, salt string) bool {
	sign := Sign(data, salt)
	if strings.Compare(sign_str, sign) > -1 {
		return true
	}
	return false
}
